<!DOCTYPE gui>
<gui name="korganizer" version="432"  translationDomain="korganizer">
  <MenuBar>
    <Menu name="file"><text>&amp;File</text>
      <Menu name="import"><text>&amp;Import</text>
        <Action name="import_icalendar"/>
        <Action name="import_ical"/>
        <Merge/>
        <Separator/>
        <Action name="downloadnewstuff"/>
      </Menu>
      <Menu name="export"><text>&amp;Export</text>
        <Action name="export_icalendar"/>
        <Merge/>
        <Separator/>
        <Action name="uploadnewstuff"/>
      </Menu>
      <Separator/>
      <Action name="file_archive"/>
      <Action name="purge_completed"/>
      <Separator/>
    </Menu>
    <Menu name="edit"><text>&amp;Edit</text>
      <Merge/>
      <Separator/>
      <Action name="edit_delete" append="edit_paste_merge"/>
    </Menu>
    <Menu name="view"><text>&amp;View</text>
      <Action name="view_agenda"/>
      <Action name="view_month"/>
      <Action name="view_timeline"/>
      <Action name="view_todo"/>      
      <Action name="view_list"/>
      <Action name="view_whatsnext"/>
      <Action name="view_journal"/>
      <Merge/>
      <Separator/>
      <Action name="update"/>
      <Separator/>
      <Action name="hide_completed_todos"/>
      <Separator/>
      <Action name="filter_select" append="save_merge"/>
      <Separator/>
      <Menu name="zoom"><text>&amp;Zoom</text>
        <Action name="zoom_in_horizontally"/>
        <Action name="zoom_out_horizontally"/>
        <Separator/>
        <Action name="zoom_in_vertically"/>
        <Action name="zoom_out_vertically"/>
      </Menu>
    </Menu>
    <Menu name="go"><text>&amp;Go</text>
      <Action name="go_previous"/>
      <Action name="go_next"/>
      <Separator/>
      <Action name="go_today"/>
      <Separator/>
      <Action name="select_day"/>
      <Action name="select_nextx"/>
      <Action name="select_workweek"/>
      <Action name="select_week"/>
      <Action name="select_newmonth"/>
      <Merge/>
    </Menu>
    <Menu name="actions"><text>&amp;Actions</text>
      <Action name="new_event"/>
      <Action name="new_todo"/>
      <Action name="new_subtodo"/>
      <Action name="new_journal"/>
      <Separator/>
      <Action name="show_incidence"/>
      <Action name="edit_incidence"/>
      <Action name="delete_incidence"/>
      <Separator/>
      <Action name="unsub_todo"/>
      <Separator/>
      <Action name="assign_resource"/>
      <Separator/>
      <Action name="activate_alarm"/>
    </Menu>
    <Menu name="schedule"><text>S&amp;chedule</text>
      <Action name="schedule_publish"/>
      <Action name="schedule_request"/>
      <Action name="schedule_reply"/>
      <Action name="schedule_cancel"/>
      <Action name="schedule_refresh"/>
      <Action name="schedule_counter"/>
      <Action name="schedule_forward"/>
      <Separator/>
      <Action name="mail_freebusy"/>
      <Action name="upload_freebusy"/>
    </Menu>
    <Menu name="settings"><text>&amp;Settings</text>
      <Menu name="sidebar" append="show_merge"><text>&amp;Sidebar</text>
        <Action name="show_datenavigator"/>
        <Action name="show_todoview"/>
        <Action name="show_eventviewer"/>
        <Action name="show_resourceview"/>
        <Action name="show_resourcebuttons" append="show_merge"/>
      </Menu>
      <Separator/>
      <Action name="conf_datetime" append="save_merge"/>
      <Action name="conf_resources" append="save_merge"/>
      <Separator/>
      <Action name="edit_filters" append="save_merge"/>
      <Action name="edit_categories" append="save_merge"/>
    </Menu>
    <Menu name="help"><text>&amp;Help</text>
      <Action name="show_intro"/>
    </Menu>
  </MenuBar>

  <ToolBar noMerge="1" name="mainToolBar"><text context="main toolbar">Main</text>
    <Action name="new_event"/>
    <Action name="new_todo"/>
    <Action name="new_journal"/>
    <!--
    <Separator/>
    <Action name="edit_find"/>
    -->
    <Separator/>
    <Action name="go_previous"/>
    <Action name="go_next"/>
    <Action name="go_today"/>
    <Action name="select_day"/>
    <Action name="select_week"/>
  </ToolBar>

  
  <ToolBar noMerge="1" name="korganizer_toolbar"><text>Views</text>
    <Action name="view_agenda"/>
    <Action name="view_month"/>
    <Action name="view_timeline"/>
    <Action name="view_todo"/>
    <Action name="view_journal"/>
    <Merge/>
  </ToolBar>

  <ToolBar noMerge="1" name="schedule_toolbar" hidden="true"><text>Schedule</text>
    <Action name="schedule_publish"/>
    <Action name="schedule_request"/>
    <Action name="schedule_reply"/>
    <Action name="addressbook"/>
  </ToolBar>

  <ToolBar noMerge="1" name="filter_toolbar" hidden="true"><text>Filters Toolbar</text>
    <Action name="filter_select"/>
    <Action name="edit_filters"/>
  </ToolBar>

  <Menu name="rmb_selection_popup">
    <Action name="new_event"/>
    <Action name="new_todo"/>
    <Action name="new_journal"/>
    <Separator/>
    <Action name="edit_paste"/>
    <Separator/>
    <Action name="configure_view"/>
    <Merge/>
  </Menu>

  <Menu name="akonadi_collectionview_contextmenu">
    <Action name="akonadi_collection_create"/>
    <Action name="akonadi_collection_delete"/>
    <Separator/>
      <Menu name="calendar_color"><text>Calendar Colors</text>
      <Action name="assign_color"/>
      <Action name="disable_color"/>
      </Menu>
    <Separator/>
    <Action name="set_standard_calendar"/>
    <Separator/>
    <Action name="akonadi_collection_move_to_menu"/>
    <Action name="akonadi_collection_copy_to_menu"/>
    <Separator/>
    <Action name="collection_disable"/>
    <Action name="collection_enable"/>
    <Separator/>
    <Action name="akonadi_resource_create"/>
    <Action name="akonadi_resource_delete"/>
    <Action name="akonadi_collection_sync"/>
    <Action name="akonadi_resource_synchronize"/>
    <Action name="akonadi_collection_sync_recursive"/>
    <Separator/>
    <Action name="akonadi_collection_properties"/>
    <Action name="akonadi_resource_properties"/>
    <Separator/>
    <Action name="serverside_subscription"/>
  </Menu>

</gui>
